from __future__ import annotations

from tigrbl_base._base._table_registry_base import TableRegistryBase


class TableRegistry(TableRegistryBase):
    """Concrete table registry implementation."""


__all__ = ["TableRegistry"]
